
use Test::More tests => 10;
BEGIN {
  use_ok('Egg');
  use_ok('Egg::Base');
  use_ok('Egg::Const');
  use_ok('Egg::Exception');
  use_ok('Egg::Helper');
  use_ok('Egg::Model');
  use_ok('Egg::View');
  use_ok('Egg::Request');
  use_ok('Egg::Response');
  use_ok('Egg::Release');
  };
